@@ -18,11 +18,13 @@
 
 use crate::property::{
     HasCharCount, HasCheckedValue, HasDecimalDigits, HasEmptyValue, HasLength, HasMember,
+    HasZeroValue,
 };
 use crate::{
     invalid_optional_value, invalid_relation, invalid_value, FieldName, RelatedFields, Validate,
     Validation, Value,
 };
+use std::convert::TryFrom;
 
 /// Error code: the value does not assert to true (`AssertTrue` constraint)
 pub const INVALID_ASSERT_TRUE: &str = "invalid-assert-true";
@@ -77,6 +79,9 @@ pub const INVALID_BOUND_OPEN_MAX: &str = "invalid-bound-open-max";
 /// (`Bound::OpenRange` or `Bound::OpenClosedRange` constraint)
 pub const INVALID_BOUND_OPEN_MIN: &str = "invalid-bound-open-min";
 
+/// Error code: the value is zero (`NonZero` constraint)
+pub const INVALID_NON_ZERO: &str = "invalid-non-zero";
+
 /// Error code: the number of integer digits is not less than or equal to the
 /// specified maximum (`Digits::integer` constraint)
 pub const INVALID_DIGITS_INTEGER: &str = "invalid-digits-integer";
@@ -93,19 +98,21 @@ pub const INVALID_CONTAINS_ELEMENT: &str = "invalid-contains-element";
 pub const INVALID_MUST_MATCH: &str = "invalid-must-match";
 
 /// Error code: the first value is not less than or equal to the second value
-/// (`FromTo::Inclusive` constraint)
-pub const INVALID_FROM_TO_INCLUSIVE: &str = "invalid-from-to-inclusive";
+/// (`MustDefineRange::Inclusive` constraint)
+pub const INVALID_MUST_DEFINE_RANGE_INCLUSIVE: &str = "invalid-must-define-range-inclusive";
 
 /// Error code: the first value is not less than the second value
-/// (`FromTo::Exclusive` constraint)
-pub const INVALID_FROM_TO_EXCLUSIVE: &str = "invalid-from-to-exclusive";
+/// (`MustDefineRange::Exclusive` constraint)
+pub const INVALID_MUST_DEFINE_RANGE_EXCLUSIVE: &str = "invalid-must-define-range-exclusive";
 
 /// The value must be true.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the
-/// [`IsChecked`](../property/trait.IsChecked.html) property trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasCheckedValue`]
+/// property trait.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasCheckedValue`]: ../property/trait.HasCheckedValue.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct AssertTrue;
 
@@ -128,10 +135,12 @@ where
 
 /// The value must be false.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the
-/// [`IsChecked`](../property/trait.IsChecked.html) property trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasCheckedValue`]
+/// property trait.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasCheckedValue`]: ../property/trait.HasCheckedValue.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct AssertFalse;
 
@@ -154,10 +163,12 @@ where
 
 /// The value must not be empty.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the
-/// [`IsEmptyValue`](../property/trait.IsEmptyValue.html) property trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasEmptyValue`]
+/// property trait.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasEmptyValue`]: ../property/trait.HasEmptyValue.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct NotEmpty;
 
@@ -185,23 +196,25 @@ where
 
 /// The length of a value must be within some bounds.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the
-/// [`HasLength`](../property/trait.HasLength.html) property trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasLength`]
+/// property trait.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasLength`]: ../property/trait.HasLength.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Length {
     /// The length of the value must be less than or equal to the specified
     /// maximum
-    Max(u32),
+    Max(usize),
     /// The length of the value must be greater than or equal to the specified
     /// minimum
-    Min(u32),
+    Min(usize),
     /// The length of the value must be between the specified minimum and
     /// maximum (inclusive)
-    MinMax(u32, u32),
+    MinMax(usize, usize),
     /// The value must be of an exact length
-    Exact(u32),
+    Exact(usize),
 }
 
 impl<T> Validate<Length, FieldName> for T
@@ -212,42 +225,39 @@ where
         let length = self.length();
         if let Some((code, expected)) = match *constraint {
             Length::Max(max) => {
-                if length > max as usize {
+                if length > max {
                     Some((INVALID_LENGTH_MAX, max))
                 } else {
                     None
                 }
             }
             Length::Min(min) => {
-                if length < min as usize {
+                if length < min {
                     Some((INVALID_LENGTH_MIN, min))
                 } else {
                     None
                 }
             }
             Length::MinMax(min, max) => {
-                if length < min as usize {
+                if length < min {
                     Some((INVALID_LENGTH_MIN, min))
-                } else if length > max as usize {
+                } else if length > max {
                     Some((INVALID_LENGTH_MAX, max))
                 } else {
                     None
                 }
             }
             Length::Exact(exact_len) => {
-                if length != exact_len as usize {
+                if length != exact_len {
                     Some((INVALID_LENGTH_EXACT, exact_len))
                 } else {
                     None
                 }
             }
         } {
-            Validation::failure(vec![invalid_value(
-                code,
-                name,
-                self.length() as u32,
-                expected,
-            )])
+            let actual = Value::try_from(length).ok();
+            let expected = Value::try_from(expected).ok();
+            Validation::failure(vec![invalid_optional_value(code, name, actual, expected)])
         } else {
             Validation::success(self)
         }
@@ -256,23 +266,25 @@ where
 
 /// The number of characters must be within some bounds.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the
-/// [`HasCharCount`](../property/trait.HasCharCount.html) property trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasCharCount`]
+/// property trait.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasCharCount`]: ../property/trait.HasCharCount.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum CharCount {
     /// The number of characters must be less than or equal to the specified
     /// maximum
-    Max(u32),
+    Max(usize),
     /// The number of characters must be greater than or equal to the specified
     /// minimum
-    Min(u32),
+    Min(usize),
     /// The number of characters must be between the specified minimum and
     /// maximum (inclusive)
-    MinMax(u32, u32),
+    MinMax(usize, usize),
     /// The number of characters must be equal to the specified amount
-    Exact(u32),
+    Exact(usize),
 }
 
 impl<T> Validate<CharCount, FieldName> for T
@@ -287,42 +299,39 @@ where
         let char_count = self.char_count();
         if let Some((code, expected)) = match *constraint {
             CharCount::Max(max) => {
-                if char_count > max as usize {
+                if char_count > max {
                     Some((INVALID_CHAR_COUNT_MAX, max))
                 } else {
                     None
                 }
             }
             CharCount::Min(min) => {
-                if char_count < min as usize {
+                if char_count < min {
                     Some((INVALID_CHAR_COUNT_MIN, min))
                 } else {
                     None
                 }
             }
             CharCount::MinMax(min, max) => {
-                if char_count < min as usize {
-                    Some((INVALID_LENGTH_MIN, min))
-                } else if char_count > max as usize {
+                if char_count < min {
+                    Some((INVALID_CHAR_COUNT_MIN, min))
+                } else if char_count > max {
                     Some((INVALID_CHAR_COUNT_MAX, max))
                 } else {
                     None
                 }
             }
             CharCount::Exact(exact_val) => {
-                if char_count != exact_val as usize {
+                if char_count != exact_val {
                     Some((INVALID_CHAR_COUNT_EXACT, exact_val))
                 } else {
                     None
                 }
             }
         } {
-            Validation::failure(vec![invalid_value(
-                code,
-                name,
-                self.char_count() as u32,
-                expected,
-            )])
+            let actual = Value::try_from(char_count).ok();
+            let expected = Value::try_from(expected).ok();
+            Validation::failure(vec![invalid_optional_value(code, name, actual, expected)])
         } else {
             Validation::success(self)
         }
@@ -331,9 +340,11 @@ where
 
 /// The value must be within some bounds.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the `PartialOrd` trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the `PartialOrd` trait
+/// and `Into<Value>`.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum Bound<T> {
     /// The value must be between the specified minimum (inclusive) and
@@ -413,12 +424,47 @@ where
     }
 }
 
+/// Values of zero are not allowed.
+///
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasZeroValue`]
+/// property trait and `Into<Value>`.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasZeroValue`]: ../property/trait.HasZeroValue.html
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct NonZero;
+
+impl<T> Validate<NonZero, FieldName> for T
+where
+    T: HasZeroValue + Into<Value>,
+{
+    fn validate(
+        self,
+        name: impl Into<FieldName>,
+        _constraint: &NonZero,
+    ) -> Validation<NonZero, Self> {
+        if self.is_zero_value() {
+            Validation::failure(vec![invalid_optional_value(
+                INVALID_NON_ZERO,
+                name,
+                Some(self.into()),
+                None,
+            )])
+        } else {
+            Validation::success(self)
+        }
+    }
+}
+
 /// Maximum number of allowed integer digits and fraction digits.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the
-/// [`DecimalDigits`](../property/trait.DecimalDigits.html) property trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasDecimalDigits`]
+/// property trait.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasDecimalDigits`]: ../property/trait.HasDecimalDigits.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct Digits {
     /// Maximum number of allowed integer digits (digits to the left of the
@@ -472,10 +518,12 @@ where
 /// The value must contain the specified member or the specified member must be
 /// part of the value.
 ///
-/// The validation function can be applied in the
-/// [`FieldName`](../core/struct.FieldName.html) context.
-/// It is implemented for all types `T` that implement the
-/// [`HasMember`](../property/trait.HasMember.html) property trait.
+/// The validation function can be applied in the [`FieldName`] context.
+/// It is implemented for all types `T` that implement the [`HasMember`]
+/// property trait and `Into<Value>`.
+///
+/// [`FieldName`]: ../core/struct.FieldName.html
+/// [`HasMember`]: ../property/trait.HasMember.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct Contains<'a, A>(pub &'a A);
 
@@ -504,9 +552,10 @@ where
 
 /// Two related fields must be equal.
 ///
-/// The validation function can be applied in the
-/// [`RelatedFields`](../core/struct.RelatedFields.html) context.
-/// It is implemented for all types `T` that implement the `Eq` trait.
+/// The validation function can be applied in the [`RelatedFields`] context.
+/// It is implemented for all types `T` that implement the `PartialEq` trait.
+///
+/// [`RelatedFields`]: ../core/struct.RelatedFields.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub struct MustMatch;
 
@@ -540,35 +589,36 @@ where
 /// range such as `valid_from` and `valid_until` or `min_salary` and
 /// `max_salary`.
 ///
-/// The validation function can be applied in the
-/// [`RelatedFields`](../core/struct.RelatedFields.html) context.
-/// It is implemented for all types `T` that implement the `PartialOrd` trait.
+/// The validation function can be applied in the [`RelatedFields`] context.
+/// It is implemented for all types `T` that implement the `PartialOrd` trait
+/// and `Into<Value`.
+///
+/// [`RelatedFields`]: ../core/struct.RelatedFields.html
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
-//TODO find better name for `FromTo`
-pub enum FromTo {
+pub enum MustDefineRange {
     /// The first value must be less than or equal to the second value
     Inclusive,
     /// The first value must be less than the second value
     Exclusive,
 }
 
-impl<T> Validate<FromTo, RelatedFields> for (T, T)
+impl<T> Validate<MustDefineRange, RelatedFields> for (T, T)
 where
     T: PartialOrd + Into<Value>,
 {
     fn validate(
         self,
         fields: impl Into<RelatedFields>,
-        constraint: &FromTo,
-    ) -> Validation<FromTo, Self> {
+        constraint: &MustDefineRange,
+    ) -> Validation<MustDefineRange, Self> {
         let RelatedFields(name1, name2) = fields.into();
         match *constraint {
-            FromTo::Inclusive => {
+            MustDefineRange::Inclusive => {
                 if self.0 <= self.1 {
                     Validation::success(self)
                 } else {
                     Validation::failure(vec![invalid_relation(
-                        INVALID_FROM_TO_INCLUSIVE,
+                        INVALID_MUST_DEFINE_RANGE_INCLUSIVE,
                         name1,
                         self.0,
                         name2,
@@ -576,12 +626,12 @@ where
                     )])
                 }
             }
-            FromTo::Exclusive => {
+            MustDefineRange::Exclusive => {
                 if self.0 < self.1 {
                     Validation::success(self)
                 } else {
                     Validation::failure(vec![invalid_relation(
-                        INVALID_FROM_TO_EXCLUSIVE,
+                        INVALID_MUST_DEFINE_RANGE_EXCLUSIVE,
                         name1,
                         self.0,
                         name2,
@@ -593,5 +643,46 @@ where
     }
 }
 
+#[cfg(feature = "regex")]
+pub use with_regex::*;
+
+#[cfg(feature = "regex")]
+mod with_regex {
+    use crate::{invalid_value, FieldName, Validate, Validation};
+    use regex::Regex;
+
+    /// Error code: the value does not match the specified pattern
+    /// (`Pattern` constraint)
+    pub const INVALID_PATTERN: &str = "invalid-pattern";
+
+    /// The value must match some regular expression.
+    ///
+    /// The validation function can be applied in the [`FieldName`] context.
+    /// It is implemented for `String`.
+    ///
+    /// [`FieldName`]: ../core/struct.FieldName.html
+    #[derive(Debug, Clone)]
+    pub struct Pattern(pub Regex);
+
+    impl Validate<Pattern, FieldName> for String {
+        fn validate(
+            self,
+            name: impl Into<FieldName>,
+            constraint: &Pattern,
+        ) -> Validation<Pattern, Self> {
+            if constraint.0.is_match(&self) {
+                Validation::success(self)
+            } else {
+                Validation::failure(vec![invalid_value(
+                    INVALID_PATTERN,
+                    name,
+                    self,
+                    constraint.0.to_string(),
+                )])
+            }
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests;
