@@ -4,9 +4,12 @@
 use bigdecimal::BigDecimal;
 #[cfg(feature = "chrono")]
 use chrono::{DateTime, NaiveDate, TimeZone, Utc};
+#[cfg(feature = "num-bigint")]
+use num_bigint::BigInt;
 #[cfg(feature = "serde1")]
 use serde::{Deserialize, Serialize};
 use std::borrow::Cow;
+use std::convert::TryFrom;
 use std::error::Error;
 use std::fmt;
 use std::fmt::{Debug, Display, Write};
@@ -449,11 +452,12 @@ impl<C, T> Validation<C, T> {
 /// Important types of 3rd party crates are supported through optional crate
 /// features:
 ///
-/// | 3rd party crate | supported type  | crate feature |
-/// |-----------------|-----------------|---------------|
-/// | [`bigdecimal`]  | `BigDecimal`    | `bigdecimal`  |
-/// | [`chrono`]      | `NaiveDate`     | `chrono`      |
-/// | [`chrono`]      | `DateTime`      | `chrono`      |
+/// | supported type | crate feature | 3rd party crate |
+/// |----------------|---------------|-----------------|
+/// | `BigInt`       | `num-bigint`  | [`num-bigint`]  |
+/// | `BigDecimal`   | `bigdecimal`  | [`bigdecimal`]  |
+/// | `NaiveDate`    | `chrono`      | [`chrono`]      |
+/// | `DateTime`     | `chrono`      | [`chrono`]      |
 ///
 /// The `From` trait is implemented for the underlying types. Additionally
 /// there are implementations of the `From` trait for the primitive types `i8`,
@@ -475,6 +479,7 @@ impl<C, T> Validation<C, T> {
 /// [`ConstraintViolation`]: enum.ConstraintViolation.html
 /// [`bigdecimal`]: https://crates.io/crates/bigdecimal
 /// [`chrono`]: https://crates.io/crates/chrono
+/// [`num-bigint`]: https://crates.io/crates/num-bigint
 #[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq)]
 pub enum Value {
@@ -499,6 +504,9 @@ pub enum Value {
     /// a value with date, time and timezone
     #[cfg(feature = "chrono")]
     DateTime(DateTime<Utc>),
+    /// a big integer value
+    #[cfg(feature = "num-bigint")]
+    BigInteger(BigInt),
 }
 
 impl Display for Value {
@@ -516,6 +524,8 @@ impl Display for Value {
             Value::Date(value) => write!(f, "{}", value),
             #[cfg(feature = "chrono")]
             Value::DateTime(value) => write!(f, "{}", value),
+            #[cfg(feature = "num-bigint")]
+            Value::BigInteger(value) => write!(f, "{}", value),
         }
     }
 }
@@ -577,7 +587,7 @@ impl From<u64> for Value {
     fn from(value: u64) -> Self {
         assert!(
             value <= i64::max_value() as u64,
-            "u64 value to big to be converted to i64"
+            "u64 value too big to be converted to i64"
         );
         Value::Long(value as i64)
     }
@@ -625,6 +635,43 @@ where
     }
 }
 
+#[cfg(feature = "num-bigint")]
+impl From<BigInt> for Value {
+    fn from(value: BigInt) -> Self {
+        Value::BigInteger(value)
+    }
+}
+
+#[cfg(target_pointer_width = "32")]
+impl TryFrom<usize> for Value {
+    type Error = &'static str;
+
+    fn try_from(value: usize) -> Result<Self, Self::Error> {
+        if value <= i32::max_value() as usize {
+            Ok(Value::Integer(value as i32))
+        } else if value as u64 <= i64::max_value() as u64 {
+            Ok(Value::Long(value as i64))
+        } else {
+            Err("usize value too big to be converted to i64")
+        }
+    }
+}
+
+#[cfg(target_pointer_width = "64")]
+impl TryFrom<usize> for Value {
+    type Error = &'static str;
+
+    fn try_from(value: usize) -> Result<Self, Self::Error> {
+        if value <= i32::max_value() as usize {
+            Ok(Value::Integer(value as i32))
+        } else if value <= i64::max_value() as usize {
+            Ok(Value::Long(value as i64))
+        } else {
+            Err("usize value too big to be converted to i64")
+        }
+    }
+}
+
 fn option_to_string<T: Display>(optional_value: Option<&T>) -> String {
     match optional_value {
         Some(value) => value.to_string(),
