@@ -438,7 +438,7 @@ mod validation {
 mod value {
     use super::*;
 
-    #[cfg(not(any(feature = "bigdecimal", feature = "chrono")))]
+    #[cfg(not(any(feature = "bigdecimal", feature = "chrono", feature = "num-bigint")))]
     #[test]
     fn exhaustive_match_over_value_variants_for_default_features() {
         fn exhaustive_match(value: Value) -> i32 {
@@ -454,7 +454,11 @@ mod value {
         assert_eq!(exhaustive_match(Value::Integer(0)), 2);
     }
 
-    #[cfg(all(feature = "bigdecimal", not(feature = "chrono")))]
+    #[cfg(all(
+        feature = "bigdecimal",
+        not(feature = "chrono"),
+        not(feature = "num-bigint")
+    ))]
     #[test]
     fn exhaustive_match_over_value_variants_with_bigdecimal_feature() {
         fn exhaustive_match(value: Value) -> i32 {
@@ -471,7 +475,11 @@ mod value {
         assert_eq!(exhaustive_match(Value::Integer(0)), 2);
     }
 
-    #[cfg(all(not(feature = "bigdecimal"), feature = "chrono"))]
+    #[cfg(all(
+        not(feature = "bigdecimal"),
+        feature = "chrono",
+        not(feature = "num-bigint")
+    ))]
     #[test]
     fn exhaustive_match_over_value_variants_with_chrono_feature() {
         fn exhaustive_match(value: Value) -> i32 {
@@ -489,7 +497,28 @@ mod value {
         assert_eq!(exhaustive_match(Value::Integer(0)), 2);
     }
 
-    #[cfg(all(feature = "bigdecimal", feature = "chrono"))]
+    #[cfg(all(
+        not(feature = "bigdecimal"),
+        not(feature = "chrono"),
+        feature = "num-bigint"
+    ))]
+    #[test]
+    fn exhaustive_match_over_value_variants_with_bigdecimal_feature() {
+        fn exhaustive_match(value: Value) -> i32 {
+            match value {
+                Value::String(_) => 1,
+                Value::Integer(_) => 2,
+                Value::Long(_) => 3,
+                Value::Float(_) => 4,
+                Value::Double(_) => 5,
+                Value::Boolean(_) => 6,
+                Value::BigInteger(_) => 10,
+            }
+        }
+        assert_eq!(exhaustive_match(Value::Integer(0)), 2);
+    }
+
+    #[cfg(all(feature = "bigdecimal", feature = "chrono", feature = "num-bigint"))]
     #[test]
     fn exhaustive_match_over_value_variants_with_bigdecimal_and_chrono_features() {
         fn exhaustive_match(value: Value) -> i32 {
@@ -503,6 +532,7 @@ mod value {
                 Value::Decimal(_) => 7,
                 Value::Date(_) => 8,
                 Value::DateTime(_) => 9,
+                Value::BigInteger(_) => 10,
             }
         }
         assert_eq!(exhaustive_match(Value::Integer(0)), 2);
@@ -576,6 +606,16 @@ mod value {
         assert_eq!(value.to_string(), "2019-08-31 12:02:59 UTC");
     }
 
+    #[cfg(feature = "num-bigint")]
+    #[test]
+    fn display_format_a_value_of_big_integer() {
+        use std::str::FromStr;
+
+        let value = Value::BigInteger(BigInt::from_str("128077101").unwrap());
+
+        assert_eq!(value.to_string(), "128077101");
+    }
+
     proptest! {
         #[test]
         fn can_convert_i8_values_into_integer_value(
@@ -668,6 +708,51 @@ mod value {
 
             prop_assert!(result.is_err());
         }
+
+        #[ignore] //TODO decide whether to keep From<u64> which might panic or support TryFrom<u64> only
+        #[test]
+        fn try_from_u64_never_panics(
+            value in any::<u64>()
+        ) {
+            let _result = Value::try_from(value);
+        }
+
+        #[test]
+        fn try_from_usize_never_panics(
+            value in any::<usize>()
+        ) {
+            let _result = Value::try_from(value);
+        }
+
+        #[cfg(any(target_pointer_width = "32", target_pointer_width = "64"))]
+        #[test]
+        fn try_from_usize_value_less_than_or_equal_i32_max(
+            value in 0..=i32::max_value()
+        ) {
+            let result = Value::try_from(value as usize);
+
+            prop_assert_eq!(result, Ok(Value::Integer(value)));
+        }
+
+        #[cfg(target_pointer_width = "64")]
+        #[test]
+        fn try_from_usize_value_greater_than_i32_max_and_less_than_or_equal_i64_max(
+            value in i64::from(i32::max_value()) + 1..=i64::max_value()
+        ) {
+            let result = Value::try_from(value as usize);
+
+            prop_assert_eq!(result, Ok(Value::Long(value)));
+        }
+
+        #[cfg(target_pointer_width = "64")]
+        #[test]
+        fn try_from_usize_value_greater_than_i64_max(
+            value in i64::max_value() as u64 + 1..=u64::max_value()
+        ) {
+            let result = Value::try_from(value as usize);
+
+            prop_assert_eq!(result, Err("usize value too big to be converted to i64"));
+        }
     }
 }
 
@@ -730,7 +815,7 @@ mod invalid_relation {
     #[test]
     fn display_format_invalid_relation_of_percent_range() {
         let invalid_relation = InvalidRelation {
-            code: "invalid-from-to".into(),
+            code: "invalid-must-define-range-inclusive".into(),
             field1: Field {
                 name: "percent_from".into(),
                 actual: Some(Value::Integer(50)),
@@ -745,7 +830,7 @@ mod invalid_relation {
 
         assert_eq!(
             invalid_relation.to_string(),
-            "invalid-from-to of percent_from which is 50 and percent_to which is 20"
+            "invalid-must-define-range-inclusive of percent_from which is 50 and percent_to which is 20"
         );
     }
 }
