@@ -561,7 +561,7 @@
 //! [`failure`]: https://crates.io/crates/failure
 //! [`serde`]: https://crates.io/crates/serde
 
-#![doc(html_root_url = "https://docs.rs/valid/0.2.0")]
+#![doc(html_root_url = "https://docs.rs/valid/0.3.0")]
 #![warn(
     bare_trait_objects,
     missing_copy_implementations,
@@ -581,6 +581,8 @@
 mod bigdecimal;
 pub mod constraint;
 mod core;
+#[cfg(feature = "num-traits")]
+mod num;
 pub mod property;
 mod std_types;
 
